using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Infragistics.Win.UltraWinToolTip;
using Infragistics.Win;

namespace UltraTimeLineView_CS
{

    /// <summary>
    /// Utilities class used to generate informative formatted Tooltips.
    /// </summary>
    public static class FormattedToolTipUtils
    {
        #region Private Members

        private static string propertyImage = string.Empty;
        private static string informationImage = string.Empty;
        private static string propertyString = "Property";
        private static string classString = "Class";

        #endregion Private Members

        #region Properties

        /// <summary>
        /// String encoding for the 'Information' image.
        /// </summary>
        private static string InformationImage
        {
            get
            {
                if (FormattedToolTipUtils.informationImage == string.Empty)
                {
                    System.Reflection.Assembly assm = System.Reflection.Assembly.GetExecutingAssembly();
                    using (System.IO.Stream stream = assm.GetManifestResourceStream("UltraTimeLineView_CS.Images.Information.png"))
                    {
                        Image image = Image.FromStream(stream);
                        FormattedToolTipUtils.informationImage = Infragistics.Win.FormattedLinkLabel.FormattedLinkEditor.EncodeImage(image);
                    }
                }
                return FormattedToolTipUtils.informationImage;
            }
        }

        /// <summary>
        /// String encoding for the 'Property' image. 
        /// </summary>
        private static string PropertyImage
        {
            get
            {
                if (FormattedToolTipUtils.propertyImage == string.Empty)
                {
                    System.Reflection.Assembly assm = System.Reflection.Assembly.GetExecutingAssembly();
                    using (System.IO.Stream stream = assm.GetManifestResourceStream("UltraTimeLineView_CS.Images.Property.png"))
                    {
                        Image image = Image.FromStream(stream);
                        FormattedToolTipUtils.propertyImage = Infragistics.Win.FormattedLinkLabel.FormattedLinkEditor.EncodeImage(image);
                    }
                }
                return FormattedToolTipUtils.propertyImage;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Generates the UltraTooltipInfo for a formatted tooltip.
        /// </summary>
        /// <param name="type">Indicates what kind of tooltip is being generated.</param>
        /// <param name="className">Name of the class containing the property for Property tooltips</param>
        /// <param name="propertyName">Name of the property for Property tooltips</param>
        /// <param name="description">The description of the property or general information</param>
        /// <returns>Returns the generated formatted tooltip</returns>
        public static UltraToolTipInfo GenerateFormattedTooltip(ResourceImageType type, string className, string propertyName, string description)
        {
            UltraToolTipInfo tipInfo = new UltraToolTipInfo();
            StringBuilder stringBuilder = new StringBuilder();
            string image = GetImageString(type);

            if (image != string.Empty)
                stringBuilder.Append("<img data='" + image + "' align='Left' height='32px' width='32px' HSpace='5' />");

            switch (type)
            {
                case ResourceImageType.Property:
                    // Property related tooltip
                    stringBuilder.Append("<b>");
                    stringBuilder.Append(FormattedToolTipUtils.classString);
                    stringBuilder.Append(": </b>");
                    stringBuilder.Append(className);
                    stringBuilder.Append("<br/>");
                    stringBuilder.Append("<b>");
                    stringBuilder.Append(FormattedToolTipUtils.propertyString);
                    stringBuilder.Append(": </b>");
                    stringBuilder.Append(propertyName);
                    if (description != string.Empty)
                        stringBuilder.Append("<p/>" + SplitString(description, 40));
                    break;
                case ResourceImageType.Information:
                    // Informational tooltip
                    stringBuilder.Append(SplitString(description, 40));
                    break;
                default:
                    break;
            }

            tipInfo.ToolTipTextStyle = ToolTipTextStyle.Formatted;
            tipInfo.ToolTipTextFormatted = stringBuilder.ToString();
            return tipInfo;
        }

        /// <summary>
        /// Inserts line breaks into formatted text.
        /// </summary>
        /// <param name="text">The text to be split.</param>
        /// <param name="length">The number of characters after which to insert a break at the first space.</param>
        /// <returns>The split string</returns>
        private static string SplitString(string text, int length)
        {
            int currentIndex = 0;
            string updatedText = text;

            while (updatedText.Length - currentIndex >= length)
            {
                currentIndex = updatedText.IndexOf(' ', currentIndex + length);
                if (currentIndex < 0)
                    break;
                updatedText = updatedText.Insert(currentIndex, "<br/>");
            }

            return updatedText;
        }

        /// <summary>
        /// Gets the encoded image string based on the provided ResourceImageType
        /// </summary>
        /// <param name="type">The provided ResourceImageType</param>
        /// <returns>The encoded image string</returns>
        private static string GetImageString(ResourceImageType type)
        {
            switch (type)
            {
                case ResourceImageType.Information:
                    return InformationImage;
                case ResourceImageType.Property:
                    return PropertyImage;
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// Sets PropertyString and ClassString (for localization purposes).
        /// </summary>
        /// <param name="classString">Localized "Class" string</param>
        /// <param name="propertyString">Localized "Property" string</param>
        public static void LocalizeStrings(string classString, string propertyString)
        {
            FormattedToolTipUtils.classString = classString;
            FormattedToolTipUtils.propertyString = propertyString;
        }

        #endregion Methods

        #region Enums

        public enum ResourceImageType
        {
            Information,
            Property,
        }

        #endregion Enums
    }
}
