using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Infragistics.Win.UltraWinSchedule;

namespace UltraTimeLineView_CS
{
    /// <summary>
    /// A utilities class to assist in the creation of some dummy WinSchedule data.
    /// </summary>
    public static class WinScheduleDataCreationUtils
    {
        #region Private Members

        private static DataSet scheduleData = null;
        private const int initialAppointmentCount = 200;
        private const int randomSeed = 0;
        private static Random rnd = new Random(randomSeed);

        #endregion Private Members

        #region Properties

        #region ScheduleData
        /// <summary>
        /// Gets a DataSet containing WinSchedule-bindable data.
        /// </summary>
        public static DataSet ScheduleData
        {
            get
            {
                if (scheduleData == null)
                {
                    scheduleData = CreateScheduleData();
                }
                return scheduleData;
            }
        }
        #endregion ScheduleData

        #endregion Properties

        #region Methods

        #region Owners
        //An Array of Dummy Owner keys
        public static string[] Owners = new string[]
			{	
				"Andrew", 
				"Bob",
                "Chris", 
                "Donna", 
                "Edward",
                "Felicia",
                "Greg",
                "Harold",
                "Iris",
                "Joe",
                "Kelly",
                "Laura",
                "Mike",
                "Nick",
                "Ophelia",
                "Peter",
                "Quentin",
                "Robert",
                "Sarah",
                "Tara",
                "Uma",
                "Victor",
                "Wendy",
                "Xavier",
                "Yale",
                "Zach",
		};
        #endregion //Owners

        #region CreateAppointmentsDataTable
        /// <summary>
        /// Create the structure of the Appointments Table
        /// </summary>
        /// <returns>Returns a DataTable formatted for the Appointent data</returns>
        private static DataTable CreateAppointmentsDataTable()
        {
            DataTable newTable = new DataTable();

            //Basic Appointment properties
            // In order to function properly, the table must contain data for 
            // Subject, StartDateTime, and EndDateTime. These three fields are 
            // required for an appointment. So all of these will have DefaultValues.
            newTable.Columns.Add("Subject", typeof(string));
            newTable.Columns["Subject"].DefaultValue = "New Appointment";
            newTable.Columns.Add("StartDateTime", typeof(DateTime));
            newTable.Columns["StartDateTime"].DefaultValue = DateTime.Now;
            newTable.Columns.Add("EndDateTime", typeof(DateTime));
            newTable.Columns["EndDateTime"].DefaultValue = DateTime.Now.AddHours(1);
            newTable.Columns.Add("Description", typeof(string));

            // All other fields are Optional.

            //Properties for AllDayEvents
            newTable.Columns.Add("AllDayEvent", typeof(bool));
            newTable.Columns["AllDayEvent"].DefaultValue = false;

            //Properties for Reminders
            newTable.Columns.Add("ReminderEnabled", typeof(bool));
            newTable.Columns["ReminderEnabled"].DefaultValue = false;
            newTable.Columns.Add("ReminderInterval", typeof(int));
            newTable.Columns.Add("ReminderUnits", typeof(int));

            //Properties when using owners
            newTable.Columns.Add("OwnerKey", typeof(string));

            //Properties when using recurring appointments. 
            newTable.Columns.Add("RecurrenceId", typeof(System.Guid));
            newTable.Columns.Add("Recurrence", typeof(Byte[]));
            newTable.Columns.Add("OriginalStartDateTime", typeof(DateTime));

            //DataKey is an extra field that can be used to link the appointment to some external data. 
            newTable.Columns.Add("DataKey", typeof(string));

            //All properties 			
            // Any other Appointment properties not covered above are stored as 
            // Binary data in the AllProperties field. This includes properties like 
            // BarColor, Location, etc. 
            newTable.Columns.Add("AllProperties", typeof(Byte[]));

            return newTable;
        }
        #endregion CreateAppointmentsDataTable

        #region CreateOwnersDataTable
        /// <summary>
        /// Create the structure of the Owners Table
        /// </summary>
        /// <returns>Returns a DataTable formatted for the Owners data</returns>
        private static DataTable CreateOwnersDataTable()
        {
            DataTable newTable = new DataTable();

            //Basic Owner properties
            // In order to function properly, the table must contain data for 
            // the Owners Key. Keys are required for an Owner. 
            newTable.Columns.Add("Key", typeof(string));

            //Other Owner properties
            newTable.Columns.Add("Name", typeof(string));
            newTable.Columns.Add("EmailAddress", typeof(string));
            newTable.Columns.Add("Visible", typeof(bool));

            //Unlike Appointments, the Owner properties are all covered by 
            //individual members. But we could save space in the database by storing
            //data as binary using AllProperties and not binding the other fields. 
            newTable.Columns.Add("AllProperties", typeof(Byte[]));
            return newTable;
        }

        #endregion CreateOwnersDataTable

        #region CreateScheduleData
        /// <summary>
        /// Populates the Appointments and Owners datatables
        /// </summary>
        /// <returns>Returns a DataSet containing the Appointment and Owners data</returns>
        private static DataSet CreateScheduleData()
        {
            DataSet newDataSet = new DataSet();

            DataTable ownersTable = CreateOwnersDataTable();
            ownersTable.TableName = "Owners";
            CreateOwners(ownersTable);
            newDataSet.Tables.Add(ownersTable);

            DataTable appointmentsTable = CreateAppointmentsDataTable();
            appointmentsTable.TableName = "Appointments";
            CreateAppointments(appointmentsTable);
            newDataSet.Tables.Add(appointmentsTable);

            return newDataSet;
        }

        #endregion CreateScheduleData

        #region CreateOwners
        /// <summary>
        /// Creates dummy Owner data
        /// </summary>
        /// <param name="ownerTable">The datatable to populate with Owner data</param>
        private static void CreateOwners(DataTable ownerTable)
        {
            ownerTable.BeginInit();
            ownerTable.Rows.Clear();
            foreach (string owner in Owners)
                ownerTable.Rows.Add(new object[] { owner, DBNull.Value, DBNull.Value, true });

            ownerTable.AcceptChanges();
            ownerTable.EndInit();
        }
        #endregion CreateOwners

        #region CreateAppointments
        /// <summary>
        /// Creats dummy Appointment data
        /// </summary>
        /// <param name="appointmentTable">The datatable to populate with Appointment data</param>
        private static void CreateAppointments(DataTable appointmentTable)
        {
            appointmentTable.BeginInit();

            appointmentTable.Rows.Clear();

            for (int i = 0; i < initialAppointmentCount; i++)
            {
                DataRow newRow = appointmentTable.NewRow();
                newRow["Subject"] = "Appointment " + i.ToString("000");

                DateTime startTime = DateTime.Now.AddDays(rnd.Next(-3, 3)).AddMinutes(rnd.Next(5, 180));
                newRow["StartDateTime"] = startTime;
                DateTime endTime = startTime.AddMinutes(rnd.Next(5, 180));
                newRow["StartDateTime"] = startTime;
                newRow["EndDateTime"] = endTime;

                int RandomOwnerIndex = rnd.Next(0, Owners.Length - 1);
                newRow["OwnerKey"] = Owners[RandomOwnerIndex];

                newRow["ReminderEnabled"] = rnd.Next(0, 2) > 0 ? true : false;
                newRow["ReminderInterval"] = 15;
                newRow["ReminderUnits"] = DisplayIntervalUnits.Minutes;

                appointmentTable.Rows.Add(newRow);
            }

            appointmentTable.AcceptChanges();

            appointmentTable.EndInit();
        }

        #endregion CreateAppointments

        #region CreateHolidays
        /// <summary>
        /// Create a list of random holidays for the current year.
        /// </summary>
        /// <param name="calendarInfo">The UltraCalendarInfo object to populate with Holidays</param>
        /// <param name="count">The number of Holidays to create</param>
        public static void CreateHolidays(UltraCalendarInfo calendarInfo, int count)
        {
            DateTime startOfYear = new DateTime(DateTime.Now.Year, 1, 1);
            List<Holiday> holidays = new List<Holiday>();

            // create random holidays this year
            for (int i = 0; i < count; i++)
            {
                int dayOfYear = rnd.Next(0, 366);
                calendarInfo.Holidays.Add(startOfYear.AddDays(dayOfYear), "Holiday #" + i.ToString());
            }
        }
        #endregion CreateHolidays

        #endregion Methods
    }
}
